#include "WiFly.h" 
#include "Credentials.h"
#include "Time.h"
#include <EEPROM.h>
#include "EEPROMAnything.h"
#include <Metro.h> 

/*************** ALARM VARIABLES ********************/

#define MEM_SIZE 1024 //EEPROM memory size for ATmega328

//RTC_DS1307 RTC;

int on_alarm_hh = 0; //set in military hour time
int on_alarm_mm = 0; //only allow 15, 30, and 45 mm

int standby_alarm_hh = 0; //set in military hour time
int standby_alarm_mm = 0; //only allow 15, 30, and 45 mm

int current_hh;
int current_mm;

int has_pressed_on = 0;
int has_pressed_standby = 0;

int input;

Metro alarm_check = Metro(5000); //how often to check the timer
Metro button_reset = Metro(70000); //prevents the buttons from firing more than once in a minute (70 seconds to be sure)

//long wTime = 1352761963;          // made up time for if doesnt work
/*************** WIFI VARIABLES ********************/

Server server(80);
int tracker = 0;
String the_val;
int counter;

/*************** SETUP ********************/

void setup() {

  pinMode(2, OUTPUT); //switch toggle
  pinMode(3, OUTPUT); //'press' toggle
  digitalWrite(3, HIGH); //allows press. initially turn off
  WiFly.begin();
  if (!WiFly.join(ssid, passphrase)) {
    while (1) {
      // Hang on failure.
    }
  }

  Serial.begin(9600);

  delay(1000);              // *needed
  //wTime = WiFly.getTime();
  long wTime = WiFly.getTime();
  delay(5000);
  Serial.print("unix time is: ");
  Serial.println(wTime);
  setTime(wTime);
  adjustTime(-18000); // Adjust the time for Eastern Time Zone during DST (-14400 during summer)
  Serial.print(hour());
  Serial.print(":");
  Serial.println(minute());
  delay(250);

  Serial.print("IP: ");
  Serial.println(WiFly.ip());

  server.begin();
  Serial.println("started server");

  //Update the stored alarm values form the EEPROM
  EEPROM_readAnything(0, on_alarm_hh);
  EEPROM_readAnything(2, on_alarm_mm);
  EEPROM_readAnything(4, standby_alarm_hh);
  EEPROM_readAnything(6, standby_alarm_mm);
  delay(500);
}

void loop() {

  //Serial.print(hour());
  //Serial.print(":");
  //Serial.println(minute());
  
  Client client = server.available();


  //DateTime now = RTC.now();
  //setTime(1352761963);
  current_hh = (hour() + 0); //addition of zero turns it into an integer
  current_mm = (minute() + 0); //addition of zero turns it into an integer

  if (button_reset.check() == 1) {
    button_reset.reset();
    has_pressed_on = 0;
    has_pressed_standby = 0;
  }



  if (alarm_check.check() == 1) { //check the time to see if it should perform an action

    alarm_check.reset(); //reset to 0.
/*
    Serial.print("current: ");
    Serial.print(current_hh);
    Serial.print(":");
    Serial.println(current_mm);

    Serial.print("On alarm: ");
    Serial.print(on_alarm_hh);
    Serial.print(":");
    Serial.println(on_alarm_mm);

    Serial.print("Standby alarm: ");
    Serial.print(standby_alarm_hh);
    Serial.print(":");
    Serial.println(standby_alarm_mm);
*/
    if(current_hh == on_alarm_hh) { //ALARM FOR TURNING ON
      if(current_mm == on_alarm_mm) {
        if(has_pressed_on == 0) {//hasn't yet pressed on
          press_on();
          has_pressed_on = 1;
        }
      }
    }

    if(current_hh == standby_alarm_hh) { //ALARM FOR STANDBY
      if(current_mm == standby_alarm_mm) {
        if(has_pressed_standby == 0) {//hasn't yet pressed standby
          press_standby();
          has_pressed_standby = 1;
        }
      }
    }
  } //close alarm_check



  if (Serial.available()) {
    input = Serial.read();
    if (input == '1') { 
      erase_memory();
    } 
    if (input == '2') { 
      program_memory();
    } 
  }



  // CLIENT CONNECTED TO WIFLY SERVER
  if (client) {
    counter = 0;

    boolean current_line_is_blank = true;

    while (client.connected()) {

      if (client.available()) {

        counter++;
        char c = client.read();
        Serial.print(c);

        if (tracker == 2) { //should consider the next value for the new ON hour
          char msg;
          on_alarm_hh = int(c-'0'); 
          Serial.print("UPDATED ON ALARM!!!!!!! to: ");
          Serial.println(on_alarm_hh);
          tracker = 0;
        }

        if (tracker == 4) { //should consider the next value for the new STANDBY hour
          standby_alarm_hh = int(c-'0') + 12;      //add 12 hrs to avoid using 2 char arrays
          Serial.print("UPDATED STANDBY ALARM!!!!!!! to: ");
          Serial.println(standby_alarm_hh);
          program_memory();
          tracker = 0;
        }

        if (counter< 30) { // if we are in the first line of the page print (where the values are)

          if ( c == 'O') { //On button pressed. v=O
            Serial.println("\nPressed On");
            press_on();
            the_val = 1;
          }
          if ( c == 'S') { //Standby button pressed. v=S
            Serial.println("\nPressed Standby");
            press_standby();
            the_val = 2;
          }
          if ( c == 'h') { //new ON hour submitted. h=# or ##
            Serial.println("\nNew On Time");
            tracker = 1;
          }
          if (c =='=' && tracker == 1) { //should start parsing for the new ON hour.
            tracker = 2; 
          }
          if ( c == 'j') { //new STANDBY hour submitted j=# or ##
            Serial.println("\nNew Standby Time");
            tracker = 3;
          }
          if (c =='=' && tracker == 3) { //should start parsing for the new STANDBY hour.
            tracker = 4; 
          }

        } //end counter<30


        if (counter > 30) { //turns parsing off
          tracker = 0; 
        }

        if (c == '\n' && current_line_is_blank) {
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println();
          client.println("<html>");
          client.println("<head>");          
          client.println("<title>Remote Remote by Tomorrow Lab</title>");
          client.println("</head>");          
          delay(10);
          client.println("<body><center>");
          delay(10);
          
          client.println("<form name=\"input\" action=\"\">");
          delay(100);

          client.println("Programmed ON time:");
          delay(10);
          client.println(on_alarm_hh);
          delay(10);
          client.println(" : 00");
          delay(10);
          client.println("<br/>");
          delay(10);

          client.println("Change ON time:<br/>");
          delay(10);
          client.println("<select name=\"h\">");
          delay(10);
          /*
          client.println("<option value=\"1\">1</option>");
           delay(10);
           client.println("<option value=\"2\">2</option>");
           delay(10);
           client.println("<option value=\"3\">3</option>");
           delay(10);
           client.println("<option value=\"4\">4</option>");
           delay(10);
           client.println("<option value=\"5\">5</option>");
           delay(10);    
           client.println("<option value=\"6\">6am</option>");
           delay(10);
           */
          client.println("<option value=\"7\">7am</option>");
          delay(10);
          client.println("<option value=\"8\">8am</option>");
          delay(10);
          client.println("<option value=\"9\">9am</option>");
          delay(10);
          /*
         client.println("<option value=\"10\">10</option>");
           delay(10);
           client.println("<option value=\"11\">11</option>");
           delay(10);
           client.println("<option value=\"12\">12 noon</option>");
           delay(10);
           client.println("<option value=\"13\">13</option>");
           delay(10);
           client.println("<option value=\"14\">14</option>");
           delay(10);
           client.println("<option value=\"15\">15</option>");
           delay(10);
           client.println("<option value=\"16\">16</option>");
           delay(10);
           client.println("<option value=\"17\">17</option>");
           delay(10);
           client.println("<option value=\"18\">18</option>");
           delay(10);
           client.println("<option value=\"19\">19</option>");
           delay(10);
           client.println("<option value=\"20\">20</option>");
           delay(10);
           client.println("<option value=\"21\">21</option>");
           delay(10);
           client.println("<option value=\"22\">22</option>");
           delay(10);
           client.println("<option value=\"23\">23</option>");
           delay(10);
           */
          client.println("</select>");
          delay(10);
          //client.println(" : 00");
          delay(10);
          client.println("<br/><br/><br/><br/>");

          client.println("Programmed STANDBY time:");
          delay(10);
          client.println(standby_alarm_hh);
          delay(10);
          client.println(" : 00<br/>");
          //delay(10);
          //client.println("<br/>");
          delay(10);
          client.println("Change STANDBY time:<br/>");
          delay(10);
          client.println("<select name=\"j\">");
          delay(10);
          /*
          client.println("<option value=\"1\">1</option>");
           delay(10);
           client.println("<option value=\"2\">2</option>");
           delay(10);
           client.println("<option value=\"3\">3</option>");
           delay(10);
           client.println("<option value=\"4\">4</option>");
           delay(10);
           client.println("<option value=\"5\">5</option>");
           delay(10);
           client.println("<option value=\"6\">6</option>");
           delay(10);
           client.println("<option value=\"7\">7</option>");
           delay(10);
           client.println("<option value=\"8\">8</option>");
           delay(10);
           client.println("<option value=\"9\">9</option>");
           delay(10);
           client.println("<option value=\"10\">10</option>");
           delay(10);
           client.println("<option value=\"11\">11</option>");
           delay(10);
           client.println("<option value=\"12\">12 noon</option>");
           delay(10);
           client.println("<option value=\"13\">1pm</option>");
           delay(10);
           client.println("<option value=\"14\">2pm</option>");
           delay(10);
           client.println("<option value=\"15\">3pm</option>");
           delay(10);
           client.println("<option value=\"16\">4pm</option>");
           delay(10);
           client.println("<option value=\"17\">5pm</option>");
           delay(10);        
           client.println("<option value=\"6\">6pm</option>");  
           delay(10);
           */
          client.println("<option value=\"7\">7pm</option>");  //doing single digits only
          delay(10);
          client.println("<option value=\"8\">8pm</option>");
          delay(10);
          client.println("<option value=\"9\">9pm</option>");
          /*
         delay(10);
           client.println("<option value=\"10\">10pm</option>");         
           delay(10);
           client.println("<option value=\"23\">11pm</option>");
           delay(10);
           */
          client.println("</select>");
          delay(10);
          //client.println(" : 00");
          delay(10);

          client.println("<input type=\"submit\" value=\"Edit times\" />");
          delay(10);
          client.println("<br/><br/><br/><br/>");

          client.println("Instant Button Press:<br/>");
          delay(10);
          client.println("<input type=\"submit\" name=\"v\" value=\"On\" />");
          delay(10);
          client.println("<input type=\"submit\" name=\"v\" value=\"Standby\" />");
          delay(10);
          client.println("<br/>Last Button Press: ");
          delay(10); 
          if (the_val == 1) {
            client.println("On"); 
          }
          if (the_val == 2) {
            client.println("Standby"); 
          }

          client.println("</form>");
          delay(10);

          delay(10);  
          client.println("</center></body></html>");
          break;
        }

        if (c == '\n') {
          current_line_is_blank = true;
        } 
        else if (c != '\r') {
          current_line_is_blank = false;
        }
      }
    }
    delay(100);
    client.stop();
  }
} //END LOOP



// FUNCTIONS
void erase_memory() {

  Serial.print("Erasing Memory ...");
  for (int i = 0; i <= MEM_SIZE; i++){
    EEPROM.write(i, 0);
  }  

  on_alarm_hh = 0;
  on_alarm_mm = 0;
  standby_alarm_hh = 0;
  standby_alarm_mm = 0;

  Serial.println("Done");
}

void program_memory() {

  EEPROM_writeAnything(0, on_alarm_hh);
  EEPROM_writeAnything(2, on_alarm_mm);
  EEPROM_writeAnything(4, standby_alarm_hh);
  EEPROM_writeAnything(6, standby_alarm_mm);
}

void press_standby() {
  digitalWrite(2, LOW); //Orange Wire - Standby BUTTON
  digitalWrite(3, LOW); //allow press
  Serial.println("2 LOW, 3 LOW");
  delay(1400);
  release_button();
  delay(1000);
  digitalWrite(2, LOW); //Orange Wire - Standby BUTTON
  digitalWrite(3, LOW); //allow press
  Serial.println("2 LOW, 3 LOW");
  delay(1400);
  release_button();
  delay(1000);
  digitalWrite(2, LOW); //Orange Wire - Standby BUTTON
  digitalWrite(3, LOW); //allow press
  Serial.println("2 LOW, 3 LOW");
  delay(1400);
  release_button();
}

void press_on() {
  digitalWrite(2, HIGH); //Brown Wire - ON BUTTON
  digitalWrite(3, LOW); //allow press
  Serial.println("2 HIGH, 3 LOW");
  delay(2000);
  release_button();
}

void release_button() {
  digitalWrite(3, HIGH); //prevent press
  digitalWrite(2, LOW); //release relay
}









